@@ -1,6 +1,7 @@
 from typing import List
 import numpy as np
-from .mexpress import native_parse, FlatEx
+
+from .mexpress import FlatEx, native_parse
 
 
 class Mexpress:
@@ -8,6 +9,7 @@ class Mexpress:
         self.flatex = flatex
         self._grad = None
         self._hess = None
+        self.n_vars = self.flatex.n_vars()
 
     def _make_grad(self) -> List[FlatEx]:
         if self._grad is None:
@@ -29,10 +31,16 @@ class Mexpress:
         grad_ = self._make_grad()
         if self._hess is None:
             self._hess = [
-                [grad_i.partial(i) for i in range(self.flatex.n_vars())] for grad_i in grad_
+                [grad_i.partial(c) for c in range(r, self.flatex.n_vars()) ] for r, grad_i in enumerate(grad_)
             ]
         x = np.array(x, dtype=np.float64)
-        return np.array([[hess_ij(x) for hess_ij in hess_i] for hess_i in self._hess])
+        hess = np.zeros((self.n_vars, self.n_vars))
+        for r in range(self.n_vars):
+            for c in range(r, self.n_vars):
+                hess[r, c] = self._hess[r][c - r](x)
+            for c in range(0, r):                
+                hess[r, c] = hess[c, r]
+        return hess
 
     def __str__(self):
         return self.flatex.unparse()
